@@ -88,8 +88,20 @@ class Duration:
         if self.m_dots > 1:
             d = d + Rat(1, self.m_nh * 4)
         return d * self.m_tuplet
+    @staticmethod
+    def new_from_rat(rat):
+        d = Duration(1, 0)
+        while d.get_rat_value() > rat:
+            d.m_nh *= 2
+        # FIXME: how many is the max number of dots? Where to define this?
+        while d.get_rat_value() < rat and d.m_dots < 7:
+            d.m_dots += 1
+        if d.get_rat_value() != rat:
+            d.m_tuplet = rat / d.get_rat_value()
+        assert d.get_rat_value() == rat
+        return d
     def clone(self):
-        return Duration(self.m_nh, self.m_dots, self.m_tuplet)
+        return Duration(self.m_nh, self.m_dots, self.m_tuplet.clone())
     def __str__(self):
         return "(Duration:%s:%idot:%s)" % (self.m_nh, self.m_dots, self.m_tuplet)
     def as_mpd_string(self):
