@@ -240,6 +240,8 @@ class Voice(HasParent):
         pass
     class NoChordsInRhythmStaffException(Exception):
         pass
+    class NotUnisonException(Exception):
+        pass
     def __init__(self, parent):
         HasParent.__init__(self, parent)
         # The timelen of the Voice
@@ -374,9 +376,9 @@ class Voice(HasParent):
             #flytt til bar-class
             delta = elem.m_duration.get_rat_value() - self.m_tdict[timepos]['elem'][0].m_duration.get_rat_value()
             def fix_ties(prev_timepos, cur_timepos, next_timepos):
-                prev_e = self.m_tdict[prev_timepos]['elem'][0]
+                prev_e = self.m_tdict[prev_timepos]['elem'][0] if prev_timepos else None
                 cur_e = self.m_tdict[cur_timepos]['elem'][0]
-                next_e = self.m_tdict[next_timepos]['elem'][0]
+                next_e = self.m_tdict[next_timepos]['elem'][0] if next_timepos else None
                 if isinstance(elem, Note):
                     t = None
                     if prev_timepos and isinstance(prev_e, Note) and prev_e.m_tieinfo in ('go', 'start'):
@@ -459,6 +461,24 @@ class Voice(HasParent):
             return v[start_i:end_i]
         else:
             return v[start_i:]
+    def get_time_pitch_list(self, bpm):
+        """
+        Return a list of tuples (pitch, duration-in-seconds) of the tones
+        and rests in the voice. -1 is used for pitch for rests.
+        """
+        ret = []
+        sec = 0.0
+        for timepos in sorted(self.m_tdict.keys()):
+            # stem is a Stem or [Rest]
+            stem = self.m_tdict[timepos]['elem']
+            if len(stem) != 1:
+                raise Voice.NotUnisonException()
+            dur = float(stem[0].m_duration.get_rat_value()) * bpm[1] / bpm[0] * 60
+            if isinstance(stem[0], Note):
+                ret.append((stem[0].m_musicalpitch.semitone_pitch(), dur))
+            elif isinstance(stem[0], Rest):
+                ret.append((-1, dur))
+        return ret
     def beam(self, notes):
         """
         Notes is a list of Note objects, once Note object from each stem
