@@ -18,8 +18,8 @@ class TestClef(unittest.TestCase):
         c = Clef("violin_8")
         self.assertEquals(c.m_octaviation, -7)
     def test_constructor_unknown_clef(self):
-        self.assertRaises(UnknownClefException, lambda : Clef("not a clefname"))
-        self.assertRaises(UnknownClefException, lambda : Clef("violin _ 8"))
+        self.assertRaises(UnknownClefException, Clef, "not a clefname")
+        self.assertRaises(UnknownClefException, Clef, "violin _ 8")
     def test_octaviation(self):
         self.assertEquals(Clef("violin").steps_to_ylinepos(0), 13)
         self.assertEquals(Clef("violin_8").steps_to_ylinepos(0), 13 - 7)
@@ -171,7 +171,11 @@ class TestMpdParser(unittest.TestCase):
     def test_partial_bar(self):
         s = r"\partial 4\staff\relative c'{ c d e }"
         score = parse_to_score_object(s)
-
+    def test_times(self):
+        t = mpd.music_to_tracklist(r"\staff{ \times 2/3{c8 d e} }")
+        self.assertEquals(t[0].str_repr(),
+            "n48 d1/12 o48 n50 d1/12 o50 n52 d1/12 o52")
+        score = parse_to_score_object(r"\staff{ \times 2/3{c8 d e} }")
 
 class TestLexer(unittest.TestCase):
     def test_simplest(self):
@@ -188,14 +192,14 @@ class TestLexer(unittest.TestCase):
         t, m = l.next()
         self.assertEquals(t, mpd.lexer.Lexer.NOTE)
         self.assertEquals(m.m_duration, mpd.duration.Duration(4, 1))
-        self.assertRaises(mpd.lexer.LexerError, lambda : l.next())
+        self.assertRaises(mpd.lexer.LexerError, l.next)
     def test_rest_dotting(self):
         l = mpd.lexer.Lexer("c4 r4. r.")
         self.assertEquals(l.next()[1].m_duration, mpd.duration.Duration(4, 0))
         t, m = l.next()
         self.assertEquals(t, mpd.lexer.Lexer.REST)
         self.assertEquals(m.m_duration, mpd.duration.Duration(4, 1))
-        self.assertRaises(mpd.lexer.LexerError, lambda : l.next())
+        self.assertRaises(mpd.lexer.LexerError, l.next)
     def test_inherited_time_dotting(self):
         l = mpd.lexer.Lexer("r4.. c")
         t, m = l.next()
@@ -232,6 +236,7 @@ class TestLexer(unittest.TestCase):
                 r"c \tupletDown d \tupletBoth e \tupletUp f",
                 r"< c4 e g >",
                 r"[ c'8 e ]",
+                r"\partial 8 c8",
                 ):
             self.assertEquals(music, mpd.lexer.Lexer.to_string(list(mpd.lexer.Lexer(music))))
     def test_set_first_pitch_ok(self):
